﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace System.Drawing
{
    static class ColorHelper
    {
        /// <summary>
        /// Convert from RGB to HSV.
        /// </summary>
        /// <param name="r">Red [0,1]</param>
        /// <param name="g">Green [0,1]</param>
        /// <param name="b">Blue [0,1]</param>
        /// <param name="h">Hue [0,360]</param>
        /// <param name="s">Saturation [0,1]</param>
        /// <param name="v">Value [0,1]</param>
        public static void RGBtoHSV(double r, double g, double b, out double h, out double s, out double v)
        {
            // http://en.wikipedia.org/wiki/HSL_and_HSV

            // Clamp the ranges
            r = Clamp(r, 0, 1);
            g = Clamp(g, 0, 1);
            b = Clamp(b, 0, 1);

            double max = Max(r, g, b);
            double min = Min(r, g, b);

            // HUE [0,360] (Note: 0 = 360)
            h = 0; // default
            if (max == min) {
                h = 0;
            } else if (max == r) {
                h = (60 * (g - b) / (max - min) + 360) % 360;
            } else if (max == g) {
                h = 60 * (b - r) / (max - min) + 120;
            } else if (max == b) {
                h = 60 * (r - g) / (max - min) + 240;
            }

            // SATURATION [0,1]
            s = 0; // default
            if (max == 0) {
                s = 0;
            } else {
                s = 1 - min / max;
            }

            // VALUE [0,1]
            v = max;
        }

        /// <summary>
        /// Convert from HSV to RGB.
        /// </summary>
        /// <param name="h">Hue [0,360]</param>
        /// <param name="s">Saturation [0,1]</param>
        /// <param name="v">Value [0,1]</param>
        /// <param name="r">Red [0,1]</param>
        /// <param name="g">Green [0,1]</param>
        /// <param name="b">Blue [0,1]</param>
        public static void HSVtoRGB(double h, double s, double v, out double r, out double g, out double b)
        {
            h = Clamp(h, 0, 360);
            s = Clamp(s, 0, 1);
            v = Clamp(v, 0, 1);

            double x = h / 60;
            int i = (int)x;
            double f = x - i;

            double p = v * (1 - s);
            double q = v * (1 - f * s);
            double t = v * (1 - (1 - f) * s);

            switch (i) {
                case 0:
                    r = v;
                    g = t;
                    b = p;
                    break;
                case 1:
                    r = q;
                    g = v;
                    b = p;
                    break;
                case 2:
                    r = p;
                    g = v;
                    b = t;
                    break;
                case 3:
                    r = p;
                    g = q;
                    b = v;
                    break;
                case 4:
                    r = t;
                    g = p;
                    b = v;
                    break;
                case 5:
                    r = v;
                    g = p;
                    b = q;
                    break;
                default:
                    r = g = b = 0;
                    break;
            }
        }

        public static Color HSVtoRGB(double h, double s, double v)
        {
            double r, g, b;
            HSVtoRGB(h, s, v, out r, out g, out b);
            return Color.FromArgb(
                (int)(r * 255),
                (int)(g * 255),
                (int)(b * 255));
        }

        #region Utilities

        public static double Clamp(double value, double min, double max)
        {
            if (value < min) return min;
            if (value > max) return max;
            return value;
        }

        public static double Max(double r, double g, double b)
        {
            double max = (r > g) ? r : g;
            return (max > b) ? max : b;
        }

        public static double Min(double r, double g, double b)
        {
            double min = (r < g) ? r : g;
            return (min < b) ? min : b;
        }

        #endregion
    }
}
