﻿using System.Collections.Generic;
namespace System
{
    public static class Maths
    {
        public const double PI = 3.1415926535897931;
        public const double E = 2.7182818284590451;

        /// <summary>
        /// Converts degrees into radians. (deg * PI / 180)
        /// </summary>
        public static double ToRad(double deg)
        {
            return deg * PI / 180.0;
        }

        /// <summary>
        /// Converts radians into degrees. (rad * 180 / PI)
        /// </summary>
        public static double ToDeg(double rad)
        {
            return rad * 180.0 / PI;
        }

        /// <summary>
        /// Clamps a value between min and max inclusive.
        /// </summary>
        public static double Clamp(double value, double min, double max)
        {
            if (value < min) return min;
            if (value > max) return max;
            return value;
        }

        /// <summary>
        /// Linear Interpolation.
        /// </summary>
        public static double Lerp(double value, double min, double max)
        {
            if (min == max) return min;
            return (value - min) / (max - min);
        }


        /// <summary>
        /// Linear Interpolation, Clamped between [min ... max].
        /// </summary>
        public static double CLerp(double value, double min, double max)
        {
            return Clamp(Lerp(value, min, max), min, max);
        }


        /// <summary>
        /// Produces a continuous list of Fibanacci numbers. eg [0, 1, 2, 3, 5, 8, ...]
        /// </summary>
        public static IEnumerable<double> Fibanacci()
        {
            double a = 0, b = 1, t;

            yield return a;
            yield return b;

            while (true) {
                t = a + b;
                a = b;
                b = t;
                yield return t;
            }
        }

        /// <summary>
        /// Produces a continuous list of values scaled to the a given value. eg [1, 2, 4, 8, ...] where initial = 1, scale = 2
        /// </summary>
        public static IEnumerable<double> Scaled(double initial, double scale)
        {
            while (true) {
                yield return initial;
                initial *= scale;
            }
        }

        /// <summary>
        /// Produces a continuous list of values scaled to the golden ratio (1.6180339887). eg [1, 1.6180339887, ...]
        /// </summary>
        public static IEnumerable<double> GoldenRatio()
        {
            return Scaled(1.0, (1.0 + Math.Sqrt(5.0)) / 2.0);
        }


        public static IEnumerable<double> Fold(double initial)
        {
            yield return initial;
            while (true) {
                yield return initial;
                initial += initial;
            }
        }





        //public static int[] foo(int count)
        //{
        //    List<int> list = new List<int>();
        //    centre(list, 0, count);
        //    //list.Add(0);
        //    //list.Add(count - 1);
        //    return list.ToArray();
        //}
        //private static void centre(List<int> list, int a, int c)
        //{
        //    int b = (c - a) / 2;
        //    if (a < b && b <= c) {
        //        list.Add(b);
        //        centre(list, a, b);
        //        centre(list, b, c);
        //    }
        //}
    }
}
