﻿using System.Collections.Generic;
using System.Threading;

namespace System
{
    /// <summary>
    /// Facilitates the execution of tasks running on multiple threads.
    /// </summary>
    static class Parallel
    {
        /// <summary>
        /// The maximum number of threads.
        /// </summary>
        public const int MAX_THREADS = 64;


        /// <summary>
        /// The action method to use the worker to do work on the job.
        /// </summary>
        public delegate void Action<W, J>(W worker, J job);


        /// <summary>
        /// Runs each worker on a seperate thread and loops through calling action on each job.
        /// All controlled via a single background thread, to make it compatable with the [STAThread] attribute.
        /// </summary>
        /// <param name="action">The action delegate method.</param>
        /// <param name="workers">The list of workers.</param>
        /// <param name="jobs">The list of jobs.</param>
        public static void RunThread<W, J>(Action<W, J> action, W[] workers, J[] jobs)
        {
            Thread thread = new Thread(delegate()
            {
                Run(action, workers, jobs);
            });
            thread.Start();
            thread.Join();
        }


        /// <summary>
        /// Runs each worker on a seperate thread and loops through calling action on each job.
        /// </summary>
        /// <param name="action">The action delegate method.</param>
        /// <param name="workers">The list of workers.</param>
        /// <param name="jobs">The list of jobs.</param>
        private static void Run<W, J>(Action<W, J> action, W[] workers, J[] jobs)
        {
            if (workers.Length == 0 || jobs.Length == 0) {

                // nothing to do

                return;

            } else if (workers.Length == 1) {

                // just the single worker

                foreach (var job in jobs) {
                    action(workers[0], job);
                }

            } else if (workers.Length > MAX_THREADS) {

                // oh no, too many workers

                throw new ArgumentOutOfRangeException(string.Format("The maximum number of workers is {0}.", MAX_THREADS));

            } else {

                // launch each worker on its own thread, and action through the job queue

                var queue = new Queue<J>(jobs);
                var resetEvents = new ManualResetEvent[workers.Length];

                for (int i = 0; i < workers.Length; i++) {
                    resetEvents[i] = new ManualResetEvent(false);
                    
                    ThreadPool.QueueUserWorkItem(delegate(object obj)
                    {
                        int index = (int)obj;

                        while (true) {
                            J job;
                            lock (queue) {
                                if (queue.Count == 0)
                                    break;
                                job = queue.Dequeue();
                            }
                            action(workers[index], job);
                        }

                        resetEvents[index].Set();
                    }, i);
                }

                WaitHandle.WaitAll(resetEvents);
            }
        }
    }
}
