﻿namespace System.Drawing
{
    /// <summary>
    /// Vector 2D type.
    /// </summary>
    [Serializable, 
    System.Xml.Serialization.XmlInclude(typeof(Point)), 
    System.Xml.Serialization.XmlInclude(typeof(PointF))]
    public struct Vector
    {
        public readonly static Vector Zero = new Vector(0, 0);
        public readonly static Vector Unit = new Vector(1, 0);


        private double x;
        private double y;


        public Vector(double x, double y)
        {
            this.x = x;
            this.y = y;
        }


        [System.Xml.Serialization.XmlAttribute()]
        public double X
        {
            get { return x; }
            set { x = value; }
        }

        [System.Xml.Serialization.XmlAttribute()]
        public double Y
        {
            get { return y; }
            set { y = value; }
        }

        public bool IsEmpty
        {
            get { return x == 0 && y == 0; }
        }


        public static Vector operator -(Vector v)
        {
            return new Vector(
                -v.x,
                -v.y);
        }

        public static Vector operator +(Vector v1, Vector v2)
        {
            return new Vector(
                v1.x + v2.x,
                v1.y + v2.y);
        }

        public static Vector operator -(Vector v1, Vector v2)
        {
            return new Vector(
                v1.x - v2.x,
                v1.y - v2.y);
        }

        public static Vector operator *(Vector v, double s)
        {
            return new Vector(v.x * s, v.y * s);
        }

        public static Vector operator *(double s, Vector v)
        {
            return new Vector(s * v.x, s * v.y);
        }

        public static Vector operator /(Vector v, double s)
        {
            return new Vector(v.x / s, v.y / s);
        }


        public static implicit operator Vector(Point p)
        {
            return new Vector(p.X, p.Y);
        }

        public static implicit operator Vector(PointF p)
        {
            return new Vector(p.X, p.Y);
        }

        public static explicit operator Point(Vector v)
        {
            return new Point((int)v.x, (int)v.y);
        }

        public static explicit operator PointF(Vector v)
        {
            return new PointF((float)v.x, (float)v.y);
        }


        public double Length()
        {
            return Math.Sqrt(x * x + y * y);
        }

        public double LengthSquared()
        {
            return x * x + y * y;
        }

        public double Angle()
        {
            return Math.Atan2(y, x);
        }

        public override string ToString()
        {
            return string.Format("({0}, {1})", x, y);
        }


        /// <summary>
        /// Returns a vector representation of a given radius and angle (radians).
        /// </summary>
        /// <param name="length">Length</param>
        /// <param name="angle">Angle in radians</param>
        /// <returns></returns>
        public static Vector FromPolar(double length, double angle)
        {
            return new Vector(
                length * Math.Cos(angle),
                length * Math.Sin(angle));
        }

        public static double Dot(Vector v1, Vector v2)
        {
            return v1.x * v2.x + v1.y * v2.y;
        }

        public static double Cross(Vector v1, Vector v2)
        {
            // returns the result of a cross product of two 
            // 3D vectors with their z values of zero.
            return v1.x * v2.y - v1.y * v2.x;
        }

        /// <summary>
        /// Calculates the smallest absolute angle between two vectors.
        /// </summary>
        /// <param name="v1"></param>
        /// <param name="v2"></param>
        /// <returns></returns>
        public static double AngleBetween(Vector v1, Vector v2)
        {
            double t = Dot(v1, v2) / (v1.Length() * v2.Length());
            if (double.IsNaN(t)) return 0; // check for divide by zero
            if (t >= 1) return 0; // t = 1; // catch rounding error close to the limit of 1
            if (t <= -1) return Math.PI; // t = -1; // catch rounding error close to the limit of -1
            return Math.Acos(t);
        }

        /// <summary>
        /// Calculates the smallest relative angle from vector 1 to vector 2.
        /// CCW is positive.
        /// CW is negative.
        /// </summary>
        /// <param name="v1"></param>
        /// <param name="v2"></param>
        /// <returns></returns>
        public static double AngleBetweenWithDirection(Vector v1, Vector v2)
        {
            return AngleBetween(v1, v2) * Math.Sign(Cross(v1, v2));
        }

        /// <summary>
        /// Rotates a vector by a given angle (radians).
        /// CCW is positive.
        /// CW is negative.
        /// </summary>
        /// <param name="vector"></param>
        /// <param name="angle"></param>
        /// <returns></returns>
        public static Vector Rotate(Vector vector, double angle)
        {
            double sin = Math.Sin(angle);
            double cos = Math.Cos(angle);
            return new Vector(
                vector.x * cos - vector.y * sin,
                vector.y * cos + vector.x * sin);
        }

        /// <summary>
        /// Rotates a vector about a given origin by a given angle (radians).
        /// CCW is positive.
        /// CW is negative.
        /// </summary>
        /// <param name="vector"></param>
        /// <param name="origin"></param>
        /// <param name="angle"></param>
        /// <returns></returns>
        public static Vector RotateAbout(Vector vector, Vector origin, double angle)
        {
            return Rotate(vector - origin, angle) + origin;
        }
    }
}
